@@ -222,6 +222,7 @@ chunk_relation_index_create(Relation htrel, Relation template_indexrel, Relation
 		ts_adjust_indexinfo_attnos(indexinfo, htrel->rd_id, chunkrel);
 
 	hypertable_id = ts_hypertable_relid_to_id(htrel->rd_id);
+	Assert(hypertable_id != INVALID_HYPERTABLE_ID);
 
 	return ts_chunk_index_create_post_adjustment(hypertable_id,
 												 template_indexrel,
@@ -524,7 +525,7 @@ chunk_index_mapping_from_tuple(TupleInfo *ti, ChunkIndexMapping *cim)
 static ScanTupleResult
 chunk_index_collect(TupleInfo *ti, void *data)
 {
-	List **mappings = data;
+	List **mappings = (List **) data;
 	ChunkIndexMapping *cim;
 	MemoryContext oldmctx;
 
@@ -559,7 +560,7 @@ ts_chunk_index_get_mappings(Hypertable *ht, Oid hypertable_indexrelid)
 					 2,
 					 chunk_index_collect,
 					 NULL,
-					 &mappings,
+					 (void *) &mappings,
 					 AccessShareLock);
 
 	return mappings;
@@ -1184,6 +1185,9 @@ Datum
 ts_chunk_index_clone(PG_FUNCTION_ARGS)
 {
 	Oid chunk_index_oid = PG_GETARG_OID(0);
+	if (!OidIsValid(chunk_index_oid))
+		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE), errmsg("invalid chunk index")));
+
 	Relation chunk_index_rel;
 	Relation hypertable_rel;
 	Relation chunk_rel;
@@ -1227,7 +1231,13 @@ Datum
 ts_chunk_index_replace(PG_FUNCTION_ARGS)
 {
 	Oid chunk_index_oid_old = PG_GETARG_OID(0);
+	if (!OidIsValid(chunk_index_oid_old))
+		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE), errmsg("invalid chunk index")));
+
 	Oid chunk_index_oid_new = PG_GETARG_OID(1);
+	if (!OidIsValid(chunk_index_oid_new))
+		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE), errmsg("invalid chunk index")));
+
 	Relation index_rel;
 	Chunk *chunk;
 	ChunkIndexMapping cim;
