@@ -4,6 +4,9 @@
  * LICENSE-APACHE for a copy of the license.
  */
 #include <postgres.h>
+#include <access/tupdesc.h>
+#include <access/xact.h>
+#include <catalog/pg_attribute.h>
 #include <catalog/pg_type.h>
 #include <executor/execPartition.h>
 #include <executor/nodeModifyTable.h>
@@ -12,6 +15,7 @@
 #include <nodes/extensible.h>
 #include <nodes/makefuncs.h>
 #include <nodes/nodeFuncs.h>
+#include <nodes/pg_list.h>
 #include <nodes/plannodes.h>
 #include <optimizer/optimizer.h>
 #include <optimizer/plancat.h>
@@ -28,6 +32,7 @@
 #include "hypertable_modify.h"
 #include "nodes/chunk_append/chunk_append.h"
 #include "nodes/chunk_dispatch/chunk_dispatch.h"
+#include "utils.h"
 
 static void fireASTriggers(ModifyTableState *node);
 static void fireBSTriggers(ModifyTableState *node);
@@ -929,7 +934,7 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 				if (unlikely(!resultRelInfo->ri_projectNewInfoValid))
 					ExecInitInsertProjection(node, resultRelInfo);
 				slot = ExecGetInsertNewTuple(resultRelInfo, context.planSlot);
-				slot = ExecInsert(&context, cds->rri, slot, node->canSetTag);
+				slot = ExecInsert(&context, resultRelInfo, cds, slot, node->canSetTag);
 				break;
 			case CMD_UPDATE:
 				/* Initialize projection info if first time for this table */
@@ -1431,8 +1436,8 @@ ExecGetUpdateNewTuple(ResultRelInfo *relinfo, TupleTableSlot *planSlot, TupleTab
  * copied and modified version of ExecInsert from executor/nodeModifyTable.c
  */
 TupleTableSlot *
-ExecInsert(ModifyTableContext *context, ResultRelInfo *resultRelInfo, TupleTableSlot *slot,
-		   bool canSetTag)
+ExecInsert(ModifyTableContext *context, ResultRelInfo *resultRelInfo, ChunkDispatchState *cds,
+		   TupleTableSlot *slot, bool canSetTag)
 {
 	ModifyTableState *mtstate = context->mtstate;
 	EState *estate = context->estate;
@@ -1447,6 +1452,36 @@ ExecInsert(ModifyTableContext *context, ResultRelInfo *resultRelInfo, TupleTable
 
 	Assert(!mtstate->mt_partition_tuple_routing);
 
+	/*
+	 * Fetch the chunk dispatch state similar to how it is done in
+	 * nodeModifyTable.c. For us, this is stored in the chunk insert state,
+	 * which we add in to the chunk dispatch state in chunk_dispatch_exec().
+	 *
+	 * We can probably improve this code by removing ChunkDispatch. It is
+	 * currently the immediate child of ModifyTable and placed between
+	 * ModifyTable and the original subplan of ModifyTable. This was
+	 * previously necessary because we didn't have our own version of
+	 * ModifyTable, but since PG14 we have our own version of
+	 * ModifyTable. This means that we can move the logic to make a
+	 * partition/chunk lookup into a separate function similar to how
+	 * ExecPrepareTupleRouting() does it in nodeModifyTable.c.
+	 *
+	 *    if (proute)
+	 *    {
+	 *        ResultRelInfo *partRelInfo;
+	 *
+	 *        slot = ExecPrepareTupleRouting(mtstate, estate, proute,
+	 *                                       resultRelInfo, slot,
+	 *                                       &partRelInfo);
+	 *        resultRelInfo = partRelInfo;
+	 *  }
+	 *
+	 * The current approach is a quick fix to avoid changing too much code at
+	 * the same time and risk introducing a bug.
+	 */
+	slot = ts_chunk_dispatch_prepare_tuple_routing(cds, slot);
+	resultRelInfo = cds->rri;
+
 	ExecMaterializeSlot(slot);
 
 	resultRelationDesc = resultRelInfo->ri_RelationDesc;
@@ -1521,10 +1556,10 @@ ExecInsert(ModifyTableContext *context, ResultRelInfo *resultRelInfo, TupleTable
 
 			if (resultRelInfo->ri_Slots == NULL)
 			{
-				resultRelInfo->ri_Slots =
-					palloc(sizeof(TupleTableSlot *) * resultRelInfo->ri_BatchSize);
-				resultRelInfo->ri_PlanSlots =
-					palloc(sizeof(TupleTableSlot *) * resultRelInfo->ri_BatchSize);
+				resultRelInfo->ri_Slots = (TupleTableSlot **) palloc(sizeof(TupleTableSlot *) *
+																	 resultRelInfo->ri_BatchSize);
+				resultRelInfo->ri_PlanSlots = (TupleTableSlot **) palloc(
+					sizeof(TupleTableSlot *) * resultRelInfo->ri_BatchSize);
 			}
 
 			/*
@@ -2358,6 +2393,38 @@ ExecOnConflictUpdate(ModifyTableContext *context, ResultRelInfo *resultRelInfo,
 		ExecWithCheckOptions(WCO_RLS_CONFLICT_CHECK, resultRelInfo, existing, mtstate->ps.state);
 	}
 
+	/*
+	 * If the target relation is using Hypercore TAM, the conflict resolution
+	 * index might point to a compressed segment containing the conflicting
+	 * row. It is possible to decompress the segment immediately so that the
+	 * update can proceed on the decompressed row.
+	 */
+	if (ts_is_hypercore_am(resultRelInfo->ri_RelationDesc->rd_rel->relam))
+	{
+		ItemPointerData new_tid;
+		int ntuples =
+			ts_cm_functions->hypercore_decompress_update_segment(resultRelInfo->ri_RelationDesc,
+																 conflictTid,
+																 existing,
+																 context->estate->es_snapshot,
+																 &new_tid);
+
+		if (ntuples > 0)
+		{
+			/*
+			 * The conflicting row was decompressed, so must update the
+			 * conflictTid to point to the decompressed row.
+			 */
+			ItemPointerCopy(&new_tid, conflictTid);
+			/*
+			 * Since data was decompressed, the command counter was
+			 * incremented to make it visible. Make sure the executor uses the
+			 * latest command ID to see the changes.
+			 */
+			context->estate->es_output_cid = GetCurrentCommandId(true);
+		}
+	}
+
 	/* Project the new tuple version */
 	ExecProject(resultRelInfo->ri_onConflict->oc_ProjInfo);
 
