@@ -48,39 +48,39 @@
 
 static const WithClauseDefinition continuous_aggregate_with_clause_def[] = {
 		[ContinuousEnabled] = {
-			.arg_name = "continuous",
+			.arg_names = {"continuous", NULL},
 			.type_id = BOOLOID,
 			.default_val = (Datum)false,
 		},
 		[ContinuousViewOptionCreateGroupIndex] = {
-			.arg_name = "create_group_indexes",
+			.arg_names = {"create_group_indexes", NULL},
 			.type_id = BOOLOID,
 			.default_val = (Datum)true,
 		},
 		[ContinuousViewOptionMaterializedOnly] = {
-			.arg_name = "materialized_only",
+			.arg_names = {"materialized_only", NULL},
 			.type_id = BOOLOID,
 			.default_val = (Datum)true,
 		},
 		[ContinuousViewOptionCompress] = {
-			.arg_name = "compress",
+			.arg_names = {"enable_columnstore", "compress", NULL},
 			.type_id = BOOLOID,
 		},
 		[ContinuousViewOptionFinalized] = {
-			.arg_name = "finalized",
+			.arg_names = {"finalized", NULL},
 			.type_id = BOOLOID,
 			.default_val = (Datum)true,
 		},
 		[ContinuousViewOptionCompressSegmentBy] = {
-			 .arg_name = "compress_segmentby",
-			 .type_id = TEXTOID,
+			.arg_names = {"segmentby", "compress_segmentby", NULL},
+			.type_id = TEXTOID,
 		},
 		[ContinuousViewOptionCompressOrderBy] = {
-			 .arg_name = "compress_orderby",
+			.arg_names = {"orderby", "compress_orderby", NULL},
 			 .type_id = TEXTOID,
 		},
 		[ContinuousViewOptionCompressChunkTimeInterval] = {
-			 .arg_name = "compress_chunk_time_interval",
+			.arg_names = {"compress_chunk_time_interval", NULL},
 			 .type_id = INTERVALOID,
 		},
 };
@@ -127,7 +127,7 @@ ts_continuous_agg_get_compression_defelems(const WithClauseResult *with_clauses)
 		{
 			Node *value = (Node *) makeString(ts_with_clause_result_deparse_value(input));
 			DefElem *elem = makeDefElemExtended(EXTENSION_NAMESPACE,
-												(char *) def.arg_name,
+												(char *) def.arg_names[0],
 												value,
 												DEFELEM_UNSPEC,
 												-1);
@@ -1635,12 +1635,12 @@ ts_continuous_agg_get_query(ContinuousAgg *cagg)
 	Query *cagg_view_query;
 
 	/*
-	 * Get the direct_view definition for the finalized version because
+	 * Get the partial_view definition for the finalized version because
 	 * the user view doesn't have the "GROUP BY" clause anymore.
 	 */
 	if (ContinuousAggIsFinalized(cagg))
-		cagg_view_oid = ts_get_relation_relid(NameStr(cagg->data.direct_view_schema),
-											  NameStr(cagg->data.direct_view_name),
+		cagg_view_oid = ts_get_relation_relid(NameStr(cagg->data.partial_view_schema),
+											  NameStr(cagg->data.partial_view_name),
 											  false);
 	else
 		cagg_view_oid = ts_get_relation_relid(NameStr(cagg->data.user_view_schema),
